// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <util.h>
//#include <zip.h>
#include "buffer.h"
#include "true_false.h"
#include "code_ctrl.h"
#ifdef enable_lock
#include "./plugins/lock/lock.h"
#endif

void buffer_zip_set_name(struct buffer *in,char * name)
{
	strcpy(in->zip_file_name,name);
	in->write_to_zip=TRUE;
}

void buffer_init(struct buffer *in)
{
in->write_to_zip=FALSE;
in->norm_x_axis=FALSE;
in->norm_y_axis=FALSE;
}

void buffer_malloc(struct buffer *in)
{
in->len=0;
in->max_len=0;
in->max_len+=1024;
strcpy(in->title,"");
strcpy(in->type,"xy");
strcpy(in->title,"");
in->x_mul=1.0;
in->y_mul=1.0;
strcpy(in->x_label,"");
strcpy(in->y_label,"");
strcpy(in->x_units,"");
strcpy(in->y_units,"");
strcpy(in->section_one,"");
strcpy(in->section_two,"");
in->logscale_x=0;
in->logscale_y=0;
in->buf=(char*)malloc(sizeof(char)*in->max_len);
memset(in->buf, 0, in->max_len);
}

void buffer_add_xy_data(struct buffer *in,double *x, double *y, int len)
{
int i;
char string[200];
double x_out=0.0;
double y_out=0.0;
double max=0.0;
double min=0.0;

if (len>0)
{
	max=y[0];
	min=y[0];

	for (i=0;i<len;i++)
	{
		if (y[i]<min) min=y[i];
		if (y[i]>max) max=y[i];
	}


	for (i=0;i<len;i++)
	{
		x_out=x[i];
		y_out=y[i];
		if (in->norm_x_axis==TRUE)
		{
			x_out/=x[len-1];
		}

		if (in->norm_y_axis==TRUE)
		{
			y_out=(y[i]-min)/(max-min);
		}

		sprintf(string,"%le %le\n",x_out,y_out);
		buffer_add_string(in,string);
	}
}
}

void buffer_add_string(struct buffer *in,char * string)
{
int str_len=strlen(string);
int pos=in->len;
in->len+=str_len;
//the +1 accounts for the zero at the end of a string
if (in->len+100>in->max_len)
{
	in->max_len+=1024;
	in->buf=(char*)realloc((char*)in->buf,sizeof(char)*in->max_len);
}

//printf("%p %d %d\n",in->buf,in->max_len,in->len);
strcpy((char*)(in->buf+pos),string);
}

void buffer_add_info(struct buffer *in)
{
char temp[400];

buffer_add_string(in,"#opvdm\n");
sprintf(temp,"#title %s\n",in->title);
buffer_add_string(in,temp);

sprintf(temp,"#type %s\n",in->type);
buffer_add_string(in,temp);

sprintf(temp,"#x_mul %lf\n",in->x_mul);
buffer_add_string(in,temp);

sprintf(temp,"#y_mul %lf\n",in->y_mul);
buffer_add_string(in,temp);

sprintf(temp,"#x_label %s\n",in->x_label);
buffer_add_string(in,temp);

sprintf(temp,"#y_label %s\n",in->y_label);
buffer_add_string(in,temp);

sprintf(temp,"#x_units %s\n",in->x_units);
buffer_add_string(in,temp);

sprintf(temp,"#y_units %s\n",in->y_units);
buffer_add_string(in,temp);

sprintf(temp,"#logscale_x %d\n",in->logscale_x);
buffer_add_string(in,temp);

sprintf(temp,"#logscale_y %d\n",in->logscale_y);
buffer_add_string(in,temp);

sprintf(temp,"#section_one %s\n",in->section_one);
buffer_add_string(in,temp);

sprintf(temp,"#section_two %s\n",in->section_two);
buffer_add_string(in,temp);
}

void buffer_dump(char *path,char * file,struct buffer *in)
{
	FILE *out;
	out = fopena(path,file, "wb");
	fwrite(in->buf, in->len, 1, out);
	fclose(out);

}

#ifdef enable_lock
void buffer_dump_aes(char *path,char * file,struct buffer *in,char *key_text)
{
	char *data_out;
	long int data_out_len=0;
	aes(&data_out,&data_out_len,in->buf,in->len,TRUE,key_text);
	//printf("%ld %d\n",data_out_len,in->len);

	FILE *out;

	out = fopena(path,file, "wb");
	if (out==NULL)
	{
		ewe("Error writing %s/%s'",path,file);
	}

	fwrite(data_out, data_out_len, 1, out);

	fclose(out);
	free(data_out);
}
#endif

void buffer_free(struct buffer *in)
{
free(in->buf);
in->len=0;
in->max_len=0;
}
