// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <string.h>
#include "../../plugins/fit/fit.h"
#include "../../sim.h"
double lib_error(struct fitvars *config,int ext,char * path)
{
char exp_name[40];
char sim_name[40];
char out_name[40];
sprintf(exp_name,"%s./exp/pulse%d/data.dat",path,ext);
sprintf(sim_name,"%s./sim/pulse%d/pulse_i_norm.dat",path,ext);
sprintf(out_name,"%s./error_pulse_trans%d.dat",path,ext);


struct istruct exp;
struct istruct sim;
struct istruct mul_data;
inter_copy(&mul_data,&exp,TRUE);
inter_set_value(&mul_data,1.0);

double start=0.0;
double stop=0.0;
double error=0.0;
double max=0.0;
inter_load(&exp,exp_name);
inter_load(&sim,sim_name);

double min=inter_get_min(&sim);

inter_sub_double(&sim,min);
inter_sub_double(&exp,min);

inter_purge_zero(&sim);
inter_purge_zero(&exp);
min=exp.data[exp.len-1];

inter_sub_double(&sim,min);
inter_sub_double(&exp,min);

start=exp.x[0];
if (start<sim.x[0]) start=sim.x[0];
stop=exp.x[exp.len-1];
if (stop>sim.x[sim.len-1]) stop=sim.x[sim.len-1];

max=inter_norm_to_one_range(&exp,start,stop);
inter_rescale(&sim,1.0, 1.0/max);

inter_save(&exp,"./a.dat");
inter_save(&sim,"./b.dat");

error=inter_get_diff(NULL,&exp,&sim,start,stop,&mul_data);
inter_free(&exp);
inter_free(&sim);
inter_free(&mul_data);
return error;
}
