# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import sys
import os
import shutil
import signal
from util import replace_file_in_zip_archive
import subprocess
from tempfile import mkstemp
import logging
import zipfile
from encode import encode_now
from encode import inp_set_encode
from encode import inp_unset_encode


def inp_update_token_value(file_path, token, replace,line_number):
	if token=="#Tll":
		inp_update_token_value("thermal.inp", "#Tlr", replace,1)
		inp_update_token_value("dos0.inp", "#Tstart", replace,1)
		try:
			upper_temp=str(float(replace)+5)
		except:
			upper_temp="300.0"
		inp_update_token_value("dos0.inp", "#Tstop", upper_temp,1)

	path=os.path.dirname(file_path)

	zip_file_name=os.path.join(path,"sim.opvdm")

	if os.path.isfile(file_path):
		f=open(file_path, mode='rb')
    		lines = f.read()
		f.close()
	else:
		if os.path.isfile(zip_file_name):
			zf = zipfile.ZipFile(zip_file_name, 'r')
			lines = zf.read(os.path.basename(file_path))
			zf.close()
		else:
			return "0"

	lines=encode_now(lines,False)
	lines=lines.split("\n")

	for i in range(0, len(lines)):
		lines[i]=lines[i].rstrip()

	if lines[len(lines)-1]=='\n':
		del lines[len(lines)-1]

	for i in range(0, len(lines)):
		if lines[i]==token:
			lines[i+line_number]=replace
			break


	if os.path.isfile(file_path):
		fh, abs_path = mkstemp()
		dump=""
		for item in lines:
			dump=dump+item+"\n"

		dump=dump.rstrip("\n")
		dump=encode_now(dump,True)
		f=open(abs_path, mode='wb')
		lines = f.write(dump)
		f.close()

		os.close(fh)
		shutil.move(abs_path, file_path)
	else:
		replace_file_in_zip_archive(zip_file_name,os.path.basename(file_path),lines)

def inp_isfile(file_path):

	path=os.path.dirname(file_path)

	zip_file_name=os.path.join(path,"sim.opvdm")

	base_name=os.path.basename(file_path)

	ret=False

	if os.path.isfile(file_path):
		ret=True
	else:
		if os.path.isfile(zip_file_name):
			zf = zipfile.ZipFile(zip_file_name, 'r')
			if base_name in zf.namelist():
				ret=True
			zf.close()
		else:
			ret=False

	return ret



def inp_load_file(lines,file_path):

	path=os.path.dirname(file_path)

	read_lines=[]
	zip_file_name=os.path.join(path,"sim.opvdm")

	if os.path.isfile(file_path):
		f=open(file_path, mode='rb')
    		read_lines = f.read()
		f.close()
	else:
		if os.path.isfile(zip_file_name):
			zf = zipfile.ZipFile(zip_file_name, 'r')
			if zf.namelist().count(os.path.basename(file_path))>0:
				read_lines = zf.read(os.path.basename(file_path))
				zf.close()
			else:
				zf.close()
				return False
		else:
			return False

	read_lines=encode_now(read_lines,False)
	read_lines=read_lines.split("\n")

	del lines[:]

	for i in range(0, len(read_lines)):
		lines.append(read_lines[i].rstrip())

	return True

def inp_write_lines_to_file(file_path,lines):
	path=os.path.dirname(file_path)

	zip_file_name=os.path.join(path,"sim.opvdm")

	base_name=os.path.basename(file_path)

	if os.path.isfile(file_path)==True or os.path.isfile(zip_file_name)==False:
		inp_save_lines(file_path,lines)
	else:
		replace_file_in_zip_archive(zip_file_name,base_name,lines)

def inp_save_lines(file_path,lines):
	dump=""
	for item in lines:
		dump=dump+item+"\n"

	dump=dump.rstrip("\n")

	dump=encode_now(dump,True)
	f=open(file_path, mode='wb')
	lines = f.write(dump)
	f.close()

def inp_search_token_value(lines, token):

	for i in range(0, len(lines)):
		if lines[i]==token:
			return lines[i+1]

	return False

def inp_get_token_array(file_path, token):

	lines=[]
	ret=[]
	inp_load_file(lines,file_path)

	for i in range(0, len(lines)):
		if lines[i]==token:
			pos=i+1
			while (lines[pos][0]!="#"):
				ret.append(lines[pos])
				pos=pos+1

			return ret

	return False

def inp_get_token_value(file_path, token):

	lines=[]
	inp_load_file(lines,file_path)

	for i in range(0, len(lines)):
		if lines[i]==token:
			return lines[i+1]

	return "0"

