# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import socket, select, string, sys
import os
import socket
import sys
import multiprocessing
import shutil
from inp import inp_get_token_value
from listen_for_files_on_network import listen_for_files_on_network
from listen_for_files_on_network import encode_for_tcp
import atexit
from time import sleep
from util import join_path
from uuid import getnode as get_mac
from threading import Thread

def return_dir(host,remote_path,otest_path):
	client_path=join_path(otest_path,remote_path)
	print "searching",client_path,remote_path
	sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	sock.connect((host, 10001))
	sock.sendall(encode_for_tcp(remote_path))
	for path, dirs, files in os.walk(client_path):
		print path
		for file_name in files:
			#print file_name,"rod",host
			if file_name.endswith(".dat")==True or file_name.endswith(".inp")==True :
				local_file=os.path.join(path,file_name)
				target=os.path.join(path,file_name)
				if target.startswith(otest_path):
	    				target=target[len(otest_path):]

				if os.path.isfile(local_file):
					got_data=False
					try:
						sendfile = open(local_file, 'rb')
						#print "I will send",local_file,target,host
						data = sendfile.read()
						zip_data=data.encode("zlib")
						sendfile.close()
						got_data=True
					except:
						print "I could not open the file:"+local_file
						got_data=False

					if len(data)!=0 and got_data==True:
						try:
							sock.sendall(encode_for_tcp(target)+encode_for_tcp(len(zip_data)))
							sock.sendall(zip_data)
						except:
							print "I could not send the data!!!"
							return False
							#sock.close()

	sock.close()
	return True

class udp_client():

	def heart_beat(self):
		while(1):
			my_load=os.getloadavg()
			my_load=my_load[0]
			command="load#"+str(my_load)
			self.socket.sendto(command, (self.server_ip, self.tx_port))
			sleep(0.25)

	def goodbye(self):
		print "Goodbye!"
		os.system("killall go.o")
		self.socket.sendto("exit#", (self.server_ip, self.tx_port))
		self.socket.close()
		sys.exit()

	def send_init(self):
		cpus=multiprocessing.cpu_count()
		mac=':'.join('%02X' % ((get_mac() >> 8*i) & 0xff) for i in reversed(xrange(6)))
		self.socket.sendto("init#"+str(cpus)+"#"+mac, (self.server_ip, self.tx_port))

	def init(self):
		self.tx_port = 8888
		self.rx_port= 8887
		self.server_ip=inp_get_token_value("server.inp", "#server_ip")

		otest_path="/dev/shm/otest"

		if not os.path.exists(otest_path):
			os.makedirs(otest_path)

		#with open("/home/rod/share/orig/debug.dat", "w") as myfile:
		#	myfile.write("debug info")


		net_files=listen_for_files_on_network()
		net_files.init(otest_path,10000)
		net_files.respond(self.server_ip, self.tx_port)

		try:
		    self.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
		except socket.error:
		    print 'Failed to create socket'
		    sys.exit()

		try:
			self.socket.bind(('', self.rx_port))
		except socket.error , msg:
			print 'Bind failed. Error Code : ' + str(msg[0]) + ' Message ' + msg[1]
			sys.exit()

		atexit.register(self.goodbye)

		self.send_init()

		heart_beat_thread = Thread(target=self.heart_beat)
		heart_beat_thread.daemon = True
		heart_beat_thread.start()

		while(1) :
			d = self.socket.recvfrom(1024)
			data = d[0]
			addr = d[1]
			ip=addr[0]
			if not data:
				break
			print "command=",data
			if data.startswith("runjob"):
				command=data.split("#")
				job_name=command[1]
				if os.fork() == 0:
					#exe_name=os.path.abspath(otest_path+"go.o")
					#cmd=exe_name
					path=join_path(otest_path,job_name)
					print "RUNNING NOW",path
					#f = open("/home/rod/one.dat", "w")
					#f.write( exe_name+"\n"  )
					#f.write( path+"\n"  )
					#f.close()

					os.chdir(path)

					#with open("/home/rod/share/orig/debug.dat", "a") as myfile:
					#	myfile.write("RUNNING NOW'"+cmd+"' '"+path+"'")


					os.system("./go.o")
					sleep(1)
					count=0
					while (count<200):
						ret=return_dir(self.server_ip,command[1],otest_path)
						if (ret==True):
							break
						sleep(1)
						print "Failed to send resending!!!!!!"
						count=count+1

					if ret==False:
						print "Could not send data"
					else:
						path_to_delete=join_path(otest_path,command[1])
						print "Clean up "+path_to_delete
						shutil.rmtree(path_to_delete)

					self.socket.sendto("finished#"+command[1], (self.server_ip, self.tx_port))
					os._exit(0)

			if data.startswith("get_data"):
				command=data.split("#")
				job_name=command[1]
				return_dir(self.server_ip,command[1],otest_path)

			if data.startswith("clear"):
				print "I will delete",otest_path
				shutil.rmtree(otest_path)
				os.makedirs(otest_path)
				self.socket.sendto("ready#", (self.server_ip, self.tx_port))
				print "I have deleted",otest_path
			if data.startswith("killall"):
				os.system("killall go.o")
			if data.startswith("register"):
				self.server_ip=ip
				net_files.respond(self.server_ip, self.tx_port)
				self.send_init()
			if data.startswith("sleep"):
				os.system("sudo pm-suspend")
			if data.startswith("poweroff"):
				os.system("sudo poweroff")

