// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "../../dump.h"
#include "../../sim.h"
#include "pulse_voc.h"

static int glob_use_cap=0;

static double Rscope=0.0;

void pulse_voc_set_Rscope(double R)
{
Rscope=R;
}


void newton_sim_pulse_voc(struct device *in)
{
printf("Looking for Voc\n");
double C=in->C;
double clamp=0.1;
double step=0.01;
double e0;
double e1;
double i0;
double i1;
double deriv;
double Rdrain=Rscope+in->Rcontact;

solve_all(in);
i0=get_I(in);
e0=fabs(i0+in->Vapplied*(1.0/in->Rshunt-1.0/Rdrain));

in->Vapplied+=step;
solve_all(in);
i1=get_I(in);
e1=fabs(i1+in->Vapplied*(1.0/in->Rshunt-1.0/Rdrain));

deriv=(e1-e0)/step;
step=-e1/deriv;

step=step/(1.0+fabs(step/clamp));
in->Vapplied+=step;

int count=0;
int max=200;
do
{
e0=e1;
solve_all(in);
i1=get_I(in);
e1=fabs(i1+in->Vapplied*(1.0/in->Rshunt-1.0/Rdrain));
deriv=(e1-e0)/step;
step=-e1/deriv;

step=step/(1.0+fabs(step/clamp));
in->Vapplied+=step;
if (get_dump_status(dump_print_text)==TRUE)
{
	printf("%d pulse_voc find Voc Vapplied=%le step=%le error=%le\n",count,in->Vapplied,step,e1);
}
if (count>max) break;
count++;

}while(e1>1e-12);

return;
}


//    ----=----
//    |       |-----
//    |       |    |
//    |       |   ---
//    |       SC
//  Rscope    |   ---
//    |       |    |
//    |       |-----
//    |       R
//    |	      |
//    ----=----
void newton_aux_pulse_voc(struct device *in,double V,double* i,double* didv,double* didphi,double* didxil,double* didxipl,double* didphir,double* didxir,double* didxipr)
{
double C=in->C;
if (glob_use_cap==FALSE) C=0.0;
double i0=*i;
double didv0=*didv;
double didphi0=*didphi;
double didxil0=*didxil;
double didxipl0=*didxipl;
double didphir0=*didphir;
double didxir0=*didxir;
double didxipr0=*didxipr;
//I have looked at the + before the capasitance term and it is correct
*i=V+(i0+V/in->Rshunt+C*(V-in->Vapplied_last)/in->dt)*(Rscope+in->Rcontact);
*didv=1.0+(didv0+1.0/in->Rshunt+C*(1.0)/in->dt)*(Rscope+in->Rcontact);
*didphi=+didphi0*(Rscope+in->Rcontact);
*didxil=+didxil0*(Rscope+in->Rcontact);
*didxipl=+didxipl0*(Rscope+in->Rcontact);
*didphir=+didphir0*(Rscope+in->Rcontact);
*didxir=+didxir0*(Rscope+in->Rcontact);
*didxipr=+didxipr0*(Rscope+in->Rcontact);
return;
}

double newton_pulse_voc(struct device *in,int usecap)
{
double C=in->C;
in->kl_in_newton=TRUE;
solver_realloc(in);
glob_use_cap=usecap;
in->newton_aux=&newton_aux_pulse_voc;
solve_all(in);
if (glob_use_cap==FALSE) C=0.0;
return get_I(in)+in->Vapplied/in->Rshunt+C*(in->Vapplied-in->Vapplied_last)/in->dt;
}
